from bioservices.eutils import EUtils
from bioservices.uniprot import UniProt
import xml.etree.ElementTree as ET
import requests

def get_pubmed_info(pmid):
    eu = EUtils()
    r = eu.EFetch('pubmed', pmid, retmode='dict', rettype='abstract')
    article = r['PubmedArticleSet']['PubmedArticle']['MedlineCitation']['Article']
    title = article['ArticleTitle']
    authors_list = [a['LastName']+ ' ' + a['Initials'] for a in article['AuthorList']['Author']]
    authors = ', '.join(authors_list)
    journal_name = article['Journal']['Title']
    biblio_date = article['Journal']['JournalIssue']['PubDate']
    if 'Year' in biblio_date:
        biblio_year = biblio_date['Year']
    else:
        biblio_year = biblio_date['MedlineDate'][0:3]
    return {'title':title,
            'journal_name':journal_name,
            'biblio_year':biblio_year,
            'authors_list': authors}

def get_epo_info(patent_number):
    resp = requests.get('http://ops.epo.org/3.1/rest-services/published-data/publication/docdb/{}/biblio.json'.format(patent_number))
    data = resp.json()
    exchange_doc = data['ops:world-patent-data']['exchange-documents']['exchange-document']
    if isinstance(exchange_doc, list):
        exchange_doc = exchange_doc[0]
    title = [el['$'] for el in exchange_doc['bibliographic-data']['invention-title'] if el['@lang']=='en']
    authors = [i['inventor-name']['name']['$'] for i in exchange_doc['bibliographic-data']['parties']['inventors']['inventor'] if i['@data-format']=='original']
    biblio_year = [el['date']['$'][:4] for el in exchange_doc['bibliographic-data']['publication-reference']['document-id'] if el['@document-id-type']=='epodoc'][0]
    return {'title': title,
            'journal_name': None,
            'biblio_year': biblio_year,
            'authors_list': authors}

def get_uniprot_info(uniprot_id):
    uniprot_client = UniProt()
    ns = {'u':'http://uniprot.org/uniprot'}
    resp = uniprot_client.retrieve(uniprot_id)
    recommended_name = resp.root.findall('u:entry/u:protein/u:recommendedName/u:fullName', ns)[0].text
    organism = resp.root.findall('u:entry/u:organism/u:dbReference[@type="NCBI Taxonomy"]', ns)[0].attrib['id']
    gene = resp.root.findall('u:entry/u:gene/u:name[@type="primary"]', ns)[0].text
    entry_name = resp.root.findall('u:entry/u:name', ns)[0].text
    go_els = resp.root.findall('u:entry/u:dbReference[@type="GO"]', ns)
    molecular_functions = []
    for go_el in go_els:
        term_property_value = go_el.findall('u:property[@type="term"]', ns)[0].attrib['value']
        if term_property_value[0:2]=='F:':
            molecular_functions.append('GO_'+go_el.attrib['id'][3:])
    return {'recommended_name': recommended_name,
            'organism': int(organism),
            'gene': gene,
            'entry_name': entry_name,
            'molecular_functions': molecular_functions
           }

def get_go_info(go_id):
    resp = requests.get('https://www.ebi.ac.uk/ols/api/ontologies/go/terms/http%253A%252F%252Fpurl.obolibrary.org%252Fobo%252F{}'.format(go_id))
    data = resp.json()
    label = data['label']
    return {'label': label}

def get_taxonomy_info(taxonomy_id):
    eu = EUtils()
    r = eu.EFetch('taxonomy', taxonomy_id, retmode='dict')
    scientific_name = r['TaxaSet']['Taxon']['ScientificName']
    return {'scientific_name': scientific_name}

def get_pfam_info(pfam_acc):
    resp = requests.get('http://pfam.xfam.org/family/{}?output=xml'.format(pfam_acc))
    root = ET.fromstring(resp.text)
    ns = {'pfam': 'http://pfam.xfam.org/'}
    entry = root.findall('pfam:entry', ns)[0]
    pfam_id = entry.attrib['id']
    description = entry.findall('pfam:description', ns)[0].text.strip()
    domain_family = ''
    return {'id': pfam_id,
            'description': description}
